#!/usr/bin/perl

our @options;
my @input;
my $stamp=sprintf("generated by $0 from %s %s",$ARGV[0],scalar(localtime()));

open(INPUT,$ARGV[0]) ||die "$!";

@input=<INPUT>;

eval("\@options=".join('',@input));

printf STDERR "%d options defined\n",scalar(@options);


print STDERR $usagestr;

# generate lex declarations

open(LEXI,">../src/option_keywords.lexi") || die "$!";
foreach $option (@options){
  printf LEXI "%s\t\t{return %s;}\n",$option->{key},uc($option->{key});
}

# generate yacc token declarations

open(YACCI,">../src/option_tokens.yacci") || die "$!";
foreach $option (@options){
  printf YACCI "%%token %s\n",uc($option->{key});
}
close(YACCI);

@bool_options=grep { $_->{type} eq 'bool' } @options;
@int_options=grep { $_->{type} eq 'int' } @options;
@string_options=grep { $_->{type} eq 'string' } @options;
@char_options=grep { $_->{type} eq 'char' } @options;

open(YACCI,">../src/option_rules.yacci") || die "$!";

print YACCI "bool_option_tag:\n";
foreach $option (@bool_options){
  push (@lines,sprintf("%s { \$\$=RO_%s; }",
		       uc($option->{key}),
		       uc($option->{key})));
}

print YACCI join("|\n",@lines);
print YACCI ";\n";

@lines=();
print YACCI "char_option_tag:\n";
foreach $option (@char_options){
  push (@lines,sprintf("%s { \$\$=RO_%s; }",
		       uc($option->{key}),
		       uc($option->{key})));
}

print YACCI join("|\n",@lines);
print YACCI ";\n";

@lines=();
print YACCI "int_option_tag:\n";
foreach $option (@int_options){
  push (@lines,sprintf("%s { \$\$=RO_%s; }",
		       uc($option->{key}),
		       uc($option->{key})));
}

print YACCI join("|\n",@lines);
print YACCI ";\n";
@lines=();
print YACCI "string_option_tag:\n";
foreach $option (@string_options){
  push (@lines,sprintf("%s { \$\$=RO_%s; }",
		       uc($option->{key}),
		       uc($option->{key})));
}

print YACCI join("|\n",@lines);
print YACCI ";\n";



close(YACCI);

@lines=();
$getoptstr="i:o:r";

open(HEADER,">../src/runopts_auto.h") || die("$!");

print HEADER "/*$stamp*/\n";
print HEADER "#ifndef RUNOPTS_AUTO_H
#define RUNOPTS_AUTO_H

typedef enum { 
";
foreach $option (@options){
  $getoptstr .= $option->{optchar};
  $getoptstr .= ':' if ($option->{type} ne 'bool');
  push(@lines,sprintf("RO_%s",uc($option->{key})));
}

print HEADER join(",\n",@lines);
print HEADER "} runopt_t;\n";
print HEADER "#define RO_GETOPT_STR \"$getoptstr\"\n";
print HEADER "#endif\n";
close HEADER;

open (C,">../src/option_funcs.c") || die ("$!");

print C "/*$stamp*/\n";


@lines=();
for $option (@options){
  if ($option->{type} eq 'bool'){
    $type='';
  } else {
    $type=' '.$option->{type};
  }
  push(@lines,sprintf("-%s%s\t(%s)\t%s",
		       $option->{optchar},$type,$option->{key},
		       $option->{msg}));
  if (defined($option->{vals})){
    foreach $val (sort keys(%{$option->{vals}})){
      my $format="    %d\t%s";
      $format="    %s\t%s" if ($option->{type} eq 'char');
      printf STDERR "$val %s",${$option->{vals}}{$val};
      push(@lines,sprintf($format,$val,${$option->{vals}}{$val}));
    }
  }

}

print C "char *usage_msg[]={\n";
foreach (@lines) {
  printf C "\"%s\",\n",$_;
}
print C "NULL};\n";
print C 'void ro_set_int(runopt_t option, int val){
  if (params==0)
    err_fatal(HERE,"must call ro_init first");

  switch(option){
';
foreach $option (@int_options,@bool_options,@char_options){
  printf C "  case RO_%s:\n",uc($option->{key});
  printf C "     params->%s=val;\n",$option->{key};
  printf C "     break;\n";
}


 print C '
  default:
    err_fatal(HERE,"unknown integer option %d",option);
  }
}
';

print C "void ro_set_defaults(params_p params,startopt_t startopt){\n";

foreach $option (@bool_options,@int_options){
  if (defined($option->{default})) {
    printf C "\tparams->%s=%s;\n",$option->{key}, $option->{default};
  }
}

foreach $option (@char_options){
  if (defined($option->{default})) {
    printf C "\tparams->%s='%s';\n",$option->{key}, $option->{default};
  }
}

foreach $option (@string_options){
  if (defined($option->{default})) {
    printf C "\tstrncpy(params->%s,\"%s\",STRING_LENGTH);\n",$option->{key},
      $option->{default};
  }
}
print C "};\n";


print C 'void ro_set_string(runopt_t option, char *val){
  if (params==0)
    err_fatal(HERE,"must call ro_init first");

  switch(option){
';
foreach $option (@string_options){
  printf C "  case RO_%s:\n",uc($option->{key});
  printf C "     strncpy(params->%s,val,STRING_LENGTH);\n",$option->{key};
  printf C "     break;\n";
}
 print C '
  default:
    err_fatal(HERE,"unknown string option %d",option);
  }
}
';

print C 'bool ro_process_auto_arg(int optc, char *optarg){

  switch(optc){
';
foreach $option (@int_options){
  printf C "  case '%s':\n",$option->{optchar};
  printf C "     ro_set_int(RO_%s,atoi(optarg));\n",uc($option->{key});
  printf C "     break;\n";
}

foreach $option (@char_options){
  printf C "  case '%s':\n",$option->{optchar};
  printf C "     ro_set_int(RO_%s,optarg[0]);\n",uc($option->{key});
  printf C "     break;\n";
}

foreach $option (@bool_options){
  printf C "  case '%s':\n",$option->{optchar};
  printf C "     ro_set_int(RO_%s,1);\n",uc($option->{key});
  printf C "     break;\n";
}
foreach $option (@string_options){
  printf C "  case '%s':\n",$option->{optchar};
  printf C "     ro_set_string(RO_%s,optarg);\n",uc($option->{key});
  printf C "     break;\n";
}

 print C '
  default:
    return FALSE;
  }
  return TRUE;
}
';



close(C);


open(HEADER,">../src/option_params.h") || die("$!");

foreach $option (@options){
  printf HEADER "\t%s\t%s;\n",$option->{type},$option->{key}
}




