/**
 * @file   array_backup.c
 * @author David Bremner <bremner@mathematik.tu-muenchen.de>
 * @date   Thu Apr  3 20:03:57 2003
 * 
 * @brief  save and restore the state of an array of integers.
 * 
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include "array_backup.h"
#include "memalloc.h"
#include "error_message.h"

array_backup_p ab_new(int size){
  array_backup_p new;
  int i;

  new=memalloc(sizeof(array_backup_t));

  if (size<=0){
    size=AB_MINIMUM_STACK;
  }
  new->stack_size=size;

  new->stack=memalloc(sizeof(key_val_t)*size);
  err_assert(new->stack);
  new->stack_top=0;
  new->strategy=AB_STACK;
  new->next=NULL;
  return new;
}


void ab_destroy(array_backup_p back){
  memfree(back->stack);
  memfree(back);
}


void ab_restore8(array_backup_p back, int8_t *array){

  int t=back->stack_top;
  
  while(t>0){
    array[back->stack[t-1].key]=back->stack[t-1].val;
    t--;
  }
}

int ab_capacity(ab_p back){
  return back->stack_size;
}

int ab_fill(ab_p back){
  return back->stack_top;
}

void ab_report(ab_p back){
  fprintf(stderr,"array backup %d %d\n",back->stack_top,back->stack_size);
}
void ab_dump(array_backup_p back){

  int t=back->stack_top;
  
  while(t>0){
    printf("%d %d\n",back->stack[t-1].key,back->stack[t-1].val);
    t--;
  }
}

/** 
 * restore to an array of 16 bit integers
 * 
 * @param back what to restore from
 * @param array where to restore
 */
void ab_restore16(array_backup_p back, int16_t *array){
  int t=back->stack_top;
  
  while(t>0){
    array[back->stack[t-1].key]=back->stack[t-1].val;
    t--;
  }

}

/** 
 * restore to an array of 32 bit integers
 * 
 * @param back what to restore from
 * @param array where to restore
 */

void ab_restore32(array_backup_p back, int32_t *array){
  int t=back->stack_top;
  
  while(t>0){
    array[back->stack[t-1].key]=back->stack[t-1].val;
    t--;
  }
}


static void grow_stack(array_backup_p back){

  back->stack_size *= AB_STACK_MUL;
  back->stack_size += AB_STACK_ADD;
  back->stack=memrealloc(back->stack,back->stack_size*sizeof(key_val_t));
}


/** 
 * save one value 
 * 
 * @param back where to save
 * @param key what index to save
 * @param val old value
 */
void ab_set(array_backup_p back,int key, int val){

  int t=back->stack_top;

  if (back->stack_top>=back->stack_size)
    grow_stack(back);

  back->stack[t].val=val;
  back->stack[t].key=key;

  back->stack_top++;
}
  
