
#include <stdio.h>
#include "parser.h"

#include "combinat.h"
#include "gparams.h"
#include "nu_sign_set.h"
#include "startopt.h"
#include "statearray.h"
#include "runtime_options.h"
#include "output.h"
#include "memalloc.h"
#include "error_message.h"
#include "facet_list.h"

static void process_basis(facet_p F, int total_elements,int basis[], int side, int new_element, sarray_p chi);
static void all_bases(facet_p F, int total_elements, int side, int new_element, sarray_p chi);

#include "statearray.h"
#include "profile.h"
int main(int argc, char **argv){

  params_p params;
  sarray_p chi,bdy=0;
  facet_p F,G;
  facet_p stellation_facet;
  int stellation_elements=0;
  int bases,facets;
  int i;
  array_index_t new_element;
  natural_t *ridge;
  facet_list_p ridge_list;
  int *indices;


  params=malloc(sizeof(params_t));
  prof_init();
  ro_process_args(argc,argv,params,SO_nonuniform);
  if (!parse_input(params))
    exit(0);


  for (F=fl_first(parser.facets); F; F=fl_next(F)){
    if (F->options & FO_STELLATE){
      if (stellation_elements)
	err_fatal(HERE,"Only single stellation facet supported\n");
      F->sign*=-1;
      stellation_elements++;
      stellation_facet=F;
    }
  }



  printf("# %d stellation elements added\n",stellation_elements);

  params->n+=stellation_elements;
  new_element=params->n;
  ridge_list=fl_new(params->r,params->n);
  indices=malloc(params->r*sizeof(int));

  bases=binomial(params->n,params->r);

  chi=sa_new(bases,bases);
  for (i=0; i < bases; i++){
    sa_init(chi,i,SIGN_ANY);
  }




  for (i=0; i <nss_count(parser.basis_signs); i++){
    int old_bindex,new_bindex;
    nu_sign_t sigma;
    nu_signed_index_t si=nss_next(parser.basis_signs);
    
    old_bindex=nsi_index(si);
    which_kset(params->n-stellation_elements,params->r,old_bindex,indices);
    new_bindex=kset_index(params->n,params->r,indices);
    
    sigma=nsi_sign(si);
    sa_init(chi,new_bindex,sigma);
  }





  /* First set the signs that follow from the new point being 
     outside exactly one facet.
  */
  for (G=fl_first(parser.facets); G; G=fl_next(G)){
    if (G->facet_id != stellation_facet->facet_id){
      all_bases(G,params->n,1,new_element,chi);
    }
  }
  all_bases(stellation_facet,params->n,-1,new_element,chi);

  ridge=memalloc((params->n)*sizeof(natural_t));


  for (G=fl_first(parser.facets); G; G=fl_next(G)){
    if (G->facet_id != stellation_facet->facet_id){
	  int count;
	  count=set_intersect(stellation_facet->size,
			      stellation_facet->index_set,
			      G->size,
			      G->index_set,
			      ridge);
	  if (count>= G->rank-1){
	    ridge[count]=new_element;
	    fl_add_vlist(ridge_list,1, count+1,
			 ridge,0);
	  }
    }
  }

  memfree(ridge);

  
  
  output_start_om(params);
  output_chi(params,chi,0);
  printf("boundary={\n");
  fl_output(parser.facets);
  fl_output(ridge_list);
  printf("}\n");
  return 0;
}

static void process_basis(facet_p F,
			  int total_elements,
			  int basis[], 
			  int side,
			  int new_element,
			  sarray_p chi){
  int i;
  int nonzero_count=0;

  for (i=1; i<=F->elements; i++){
    if (!set_contains(F->size,F->index_set,i)){
      array_index_t b;
      nu_sign_t sigma;
      basis[F->rank-1]=i;
      
      b=kset_cindex(total_elements,F->rank,basis);
      sigma=sign_mulint(sa_read(chi,b),basis_parity(F->rank,basis));

      if (sign_subset(sigma,SIGN_NONZERO)){
	int new_bindex;
	basis[F->rank-1]=new_element;

	new_bindex=kset_cindex(total_elements,F->rank,basis);
	sa_set(chi,new_bindex,
	       sign_mulint(sigma,side*basis_parity(F->rank,basis)));
	nonzero_count++;     
      }
    
    }
  }
  if (nonzero_count==0){
    array_index_t new_bindex;
    basis[F->rank-1]=new_element;
    new_bindex=kset_cindex(total_elements,F->rank,basis);
    sa_set(chi,new_bindex,SIGN_ZERO);
  }
}
  
  
static void all_bases(facet_p F, int total_elements,
		      int side, 
		      int new_element, 
		      sarray_p chi){
  int *indices;
  int *basis;
  int i;

  indices=malloc(F->rank*sizeof(int));
  basis=malloc(F->rank*sizeof(int));

  /* look at all possible rank-1 bases for the facet */
    first_kset(F->size,F->rank-1,indices);
  do {
    for (i=0; i<F->rank-1; i++)
      basis[i]=F->index_set[indices[i]-1];

    process_basis(F,total_elements,basis,side,new_element,chi);
  } while (next_kset(F->size,F->rank-1,indices));
}
  
