/* vc_io.c (Vertex cover problem)

   Ambros Marzetta 94-11-20
   For more information see our WWW page at
   
         http://wwwjn.inf.ethz.ch/ambros/zram.html
	 
 * $Source$
 * $Revision: 2732 $
 * $Date: 2000-11-15 17:14:52 +0100 (Wed, 15 Nov 2000) $
 * $Log$
 * Revision 1.2  2000/11/15 16:14:52  bremner
 * committing patched version of zram source
 *
 * Revision 1.1  1998/04/15  08:47:29  marzetta
 * Initial revision
 *
 *
 */

#include <stdlib.h>		/* exit() */
#include "vc_io.h"

#define MAXHEADERLINE 256

vc_GlobalP vc_ReadGraph(FILE *infile)
{
  /* Read Graph from infile and allocate a node */
  char s[MAXHEADERLINE], c;
  int points, edges, actpt, matches, line, i, nr, other, degree, index;
  vc_GlobalP node;

  points = 0;
  edges = 0;
  line = 0;			/* line numbers start with 1 */
  do {
    (void) fgets(s, MAXHEADERLINE, infile);
    line ++;
    matches = sscanf(s, " Points:%d", &points);
    if (matches == 0) {
      if (sscanf(s, " Edges:%d", &edges) <= 0) 
	if (sscanf(s, " %c", &c) == 1 && c != '#') {
	  fprintf(stderr, "vc_ReadGraph: Error in line %1d\n", line);
	  return NULL;
	}
    } else if (feof(infile)) {
      fprintf(stderr, "vc_ReadGraph: End of file before line %d\n", line);
      return NULL;
    }
  } while (points==0 || edges==0);

  actpt = 0; /* points with positive degree */
  node = vc_NewFullNode((MINT) points, (MINT) edges);
  if (!node) fprintf(stderr, "vc_ReadGraph: Could not allocate node.\n");
  index = 0;			/* index into adjacency list */
  for (i=0;i<points;i++) {
    if (fscanf(infile, " P%d:", &nr) != 1 || nr != i) {
      fprintf(stderr, "vc_ReadGraph: Error with point %d\n", i);
      return NULL;		/* should deallocate */
    }
    degree = 0;
    vc_first(node)[i] = index;
    
    for(;;) {			/* for (edge) */
      matches = fscanf(infile, "%d", &other);
      if (matches != 1) break;	/* breaks at 'P' and at end of file */
      /*if (!(i%2) || !(other%2)) continue;*/ /* use only odd points */
      degree++;
      if (degree > MAXDEGREE || degree <= 0) {
	  fprintf(stderr, "vc_ReadGraph: degree of point %d is > %d\n",
		  i, MAXDEGREE);
	return NULL;		/* should deallocate */
      }
      if (other < 0 || other >= points || other == i) {
	if (other == i)
	  fprintf(stderr, "vc_ReadGraph: Unallowed loop (%d,%d)\n", i, other);
	else
	  fprintf(stderr, "vc_ReadGraph: Error with edge (%d,%d) (point not in graph)\n", i, other);
	return NULL;		/* should deallocate */
      }
      if (other < i) {		/* find inverse edge */
	MINT *adj2;
	adj2 = vc_adjPt(node, other);
	while (*adj2 != i && *adj2 != ADJ_LAST) adj2 ++;
	if (*adj2 == ADJ_LAST) {
	  fprintf(stderr, "vc_ReadGraph: Edge (%d,%d) is not symmetric.\n",
		  i, other);
	  return NULL;
	}
        *adj2 = vc_cadj(i, index);
	vc_adj(node)[index++] = vc_cadj(other, adj2-vc_adj(node));
      } else
	vc_adj(node)[index++] = other;
    } /* for (edge) */
    if (degree) actpt ++;
    vc_gdegree(node)[i] = degree;
    vc_adj(node)[index++] = ADJ_LAST;
    vc_triangle(node)[i] = -1;
  } /* for (point) */
  node->vc_numAct = actpt;
  if (index != points + edges) {
    fprintf(stderr, "vc_ReadGraph: Bad edge count\n");
    if (index > points + edges) exit(1);
    node->vc_numEdge = index - points;
  }
  return node;
} /* vc_ReadGraph */

